import greenfoot.*;

/**
 * This is MyWorld class in which game runs.
 * 
 * @author Chandrasekhar Thotakura
 * @version 1.0.0
 */
public class MyWorld  extends World
{
    /**
     * Defines the variables required in game.
     */
    private Snake snakeBody[];
    private Food food;
    private Counter score;
    private int direction;
    private boolean eaten;
    private int gameState;
    public static final int BLOCK = 15;

    /**
     * Constructor for objects of class MyWorld.
     */
    public MyWorld()
    {    
        super(20*BLOCK, 30*BLOCK, 1); // constructor of the super class
        direction = 0; // initialization
        eaten = false; // initialization
        gameState = 0; // initialization
    }

    /**
     * Game starts from here.
     */
    public void act() 
    {
        if(gameState==0) // Initialization State
        {
            setBackground("SnakeGame.jpg");
            if(Greenfoot.getKey()!=null) // checks if any key is pressed
                gameState = 1;
            return;
        }
        else if(gameState==1) // Game Active State
        {
            setBackground("bluerock.jpg");

            food = new Food(); // initialization of the food
            addObject(food, 0, 0); // adding food actor to the world

            score = new Counter("Score : "); // initialization of the food
            addObject(score, 19*BLOCK, 8); // adding food actor to the world

            snakeBody = new Snake[3]; // initialization of the Snake
            for(int i=0; i<snakeBody.length; i++)
            {
                snakeBody[i] = new Snake(i==0);
                addObject(snakeBody[i], 6*BLOCK+(snakeBody.length-i)*BLOCK, 2*BLOCK); // adding snake actor to the world
            }
            updateFoodLocation();
            gameState = 2;
            return;
        }
        else if(gameState==-1) // Game Over State
        {
            removeObjects(getObjects(null));
            setBackground("GameOver.jpg");
            addObject(score, 19*BLOCK, 8);
            return;
        }
        
        if(eaten)
            updateFoodLocation(); // updates the location of the food
        
        if(Greenfoot.isKeyDown("right")) // checks if right key is pressed
        {
            if(direction==1 || direction==3)
                direction = 0;
        }
        else if(Greenfoot.isKeyDown("down")) // checks if down key is pressed
        {
            if(direction==0 || direction==2)
                direction = 1;
        }
        else if(Greenfoot.isKeyDown("left")) // checks if left key is pressed
        {
            if(direction==1 || direction==3)
                direction = 2;
        }
        else if(Greenfoot.isKeyDown("up")) // checks if up key is pressed
        {
            if(direction==0 || direction==2)
                direction = 3;
        }
        
        int rotation = snakeBody[0].getRotation();
        int previousLocationX = snakeBody[0].getX();
        int previousLocationY = snakeBody[0].getY();
        
        snakeBody[0].setRotation(direction*90);
        snakeBody[0].move(BLOCK); // moves one step
        
        if(snakeBody[0].getX()!=previousLocationX || snakeBody[0].getY()!=previousLocationY) // Checks if the snake hits walls
        {
            for(int i=1; i<snakeBody.length; i++)
            {
                int tempRotation = snakeBody[i].getRotation();
                snakeBody[i].setRotation(rotation);

                previousLocationX = snakeBody[i].getX();
                previousLocationY = snakeBody[i].getY();
                snakeBody[i].move(BLOCK); // entire body moves one step
                rotation = tempRotation;
            }
            
            if(snakeBody[0].getX()==food.getX() && snakeBody[0].getY()==food.getY()) // Checks if the snake eats food
            {
                growSnake(previousLocationX, previousLocationY, rotation);
                updateFoodLocation();
                score.add(5);
            }
            
            for(int i=1; i<snakeBody.length; i++) // Checks if the snake eats itself
            {
                if(snakeBody[0].getX()==snakeBody[i].getX() && snakeBody[0].getY()==snakeBody[i].getY())
                {
                    gameState = -1;
                }
            }
        }
        else
        {
            direction = rotation/90;
            snakeBody[0].setRotation(direction*90);
            gameState = -1;
        }
    }
    
    /**
     * This method will update the length of the Snake.
     */
    private void growSnake(int x, int y, int rotation)
    {
        Snake s = new Snake(false);
        Snake oldSnake[] = new Snake[snakeBody.length];
        for(int i=0; i<snakeBody.length; i++)
        {
            oldSnake[i] = snakeBody[i];
        }
        snakeBody = new Snake[snakeBody.length+1];
        for(int i=0; i<snakeBody.length-1; i++)
        {
            snakeBody[i] = oldSnake[i];
        }
        snakeBody[snakeBody.length-1] = s;
        snakeBody[snakeBody.length-1].setRotation(rotation);        
        addObject(snakeBody[snakeBody.length-1], x, y); // adds new snake body actor
    }
    
    /**
     * This method will changes the place where the food is displayed.
     */
    private void updateFoodLocation()
    {
        int x=0, y=0;
        boolean overlap = true;
        eaten = false;
        
        while(overlap)
        {
            x = Greenfoot.getRandomNumber(getWidth()/BLOCK);
            y = Greenfoot.getRandomNumber(getHeight()/BLOCK);
            
            for(int i=0; i<snakeBody.length; i++)
            {
                // Condition to check food will not touch the snake
                if(x!=snakeBody[i].getX() || y!=snakeBody[i].getY())
                {
                    overlap = false;
                    break;
                }
            }
        }
        
        food.setLocation(x*BLOCK, y*BLOCK);
    }
}
